
/**
 * typed-contracts v0.0.5
 * Validation with good type inference (Flow support)
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _extendableBuiltin(cls) {
  function ExtendableBuiltin() {
    var instance = Reflect.construct(cls, Array.from(arguments));
    Object.setPrototypeOf(instance, Object.getPrototypeOf(this));
    return instance;
  }

  ExtendableBuiltin.prototype = Object.create(cls.prototype, {
    constructor: {
      value: cls,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });

  if (Object.setPrototypeOf) {
    Object.setPrototypeOf(ExtendableBuiltin, cls);
  } else {
    ExtendableBuiltin.__proto__ = cls;
  }

  return ExtendableBuiltin;
}

var valueToString = function valueToString(value) {
  if (Number.isNaN(value)) return 'NaN';
  switch (value) {
    case undefined:
      return 'undefined';
    case null:
      return 'null';
    default:
      return JSON.stringify(value) + ' (' + (typeof value === 'undefined' ? 'undefined' : _typeof(value)) + ')';
  }
};

var typesToString = function typesToString(expectedTypes) {
  return expectedTypes.slice(0, -2).concat(expectedTypes.slice(-2).join(' or ')).join(', ');
};

var ValidationError = function (_extendableBuiltin2) {
  _inherits(ValidationError, _extendableBuiltin2);

  _createClass(ValidationError, null, [{
    key: 'of',
    value: function of(valueName, value, expectedType) {
      for (var _len = arguments.length, otherExpectedTypes = Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
        otherExpectedTypes[_key - 3] = arguments[_key];
      }

      return new (Function.prototype.bind.apply(ValidationError, [null].concat([valueName, value, expectedType], _toConsumableArray(otherExpectedTypes))))();
    }
  }]);

  function ValidationError(valueName, value, expectedType) {
    _classCallCheck(this, ValidationError);

    var _this = _possibleConstructorReturn(this, (ValidationError.__proto__ || Object.getPrototypeOf(ValidationError)).call(this));

    _this.name = 'ValidationError';
    _this.valueName = valueName;
    _this.value = value;

    for (var _len2 = arguments.length, otherExpectedTypes = Array(_len2 > 3 ? _len2 - 3 : 0), _key2 = 3; _key2 < _len2; _key2++) {
      otherExpectedTypes[_key2 - 3] = arguments[_key2];
    }

    _this.expectedTypes = [expectedType].concat(otherExpectedTypes);
    _this.message = _this.valueName + ' must be ' + typesToString(_this.expectedTypes) + ', but ' + valueToString(value) + ' given';
    return _this;
  }

  return ValidationError;
}(_extendableBuiltin(TypeError));

var createContract = /* :: <T> */function createContract(validate) {
  var maybe = function maybe(valueName, value) {
    return value == null ? value : validate(valueName, value);
  };

  var optional = function optional(valueName, value) {
    return value === undefined ? value : validate(valueName, value);
  };

  var contract = function contract(valueName, value) {
    return validate(valueName, value);
  };

  contract.maybe = maybe;
  contract.optional = optional;

  contract.bindName = function (valueName) {
    var named = function named(value) {
      return validate(valueName, value);
    };
    named.maybe = function (value) {
      return maybe(valueName, value);
    };
    named.optional = function (value) {
      return optional(valueName, value);
    };
    return named;
  };

  return contract;
};

var isArray = /* :: <T> */function isArray(contract) {
  return createContract(function (valueName, value) {
    if (!Array.isArray(value)) return ValidationError.of(valueName, value, 'Array');

    for (var index = 0; index < value.length; index += 1) {
      var result = contract(valueName + '[' + index + ']', value[index]);
      if (result instanceof ValidationError) return result;
    }

    return value;
  });
};

var isLiteral = function isLiteral(expected) {
  return createContract(function (valueName, value) {
    if (value === expected) return value;
    return ValidationError.of(valueName, value, JSON.stringify(expected));
  });
};

var isNull = createContract(function (name, value) {
  return value === null ? value : ValidationError.of(name, value, 'null');
});

var isNumber = createContract(function (name, value) {
  return typeof value === 'number' ? value : ValidationError.of(name, value, 'number');
});

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var isObject =
/* :: <S: { [prop: string]: (valueName: string, value: mixed) => any }> */function isObject(spec) {
  return createContract(function (valueName, value) {
    if (!value || (typeof value === 'undefined' ? 'undefined' : _typeof$1(value)) !== 'object' || Array.isArray(value)) return ValidationError.of(valueName, value, 'Object');

    var keys = Object.keys(spec);

    for (var index = 0; index < keys.length; index += 1) {
      var keyName = keys[index];
      var contract = spec[keyName];
      var currentValue = value[keyName];

      var result = contract(valueName + '.' + keyName, currentValue);
      if (result instanceof ValidationError) return result;
    }

    return value;
  });
};

var isString = createContract(function (name, value) {
  return typeof value === 'string' ? value : ValidationError.of(name, value, 'string');
});

function _toConsumableArray$1(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var isUnion =
/* :: <T> */function isUnion() {
  for (var _len = arguments.length, contracts = Array(_len), _key = 0; _key < _len; _key++) {
    contracts[_key] = arguments[_key];
  }

  var unionContract = function unionContract(name, value) {
    var errors = contracts.map(function (contract) {
      return contract(name, value);
    }).reduce(function (scope, error) {
      if (error instanceof ValidationError) scope.push(error);
      return scope;
    }, []);

    if (errors.length !== contracts.length) return value;

    var types = errors.reduce(function (scope, _ref) {
      var expectedTypes = _ref.expectedTypes;

      for (var i = 0; i < expectedTypes.length; i += 1) {
        scope.push(expectedTypes[i]);
      }
      return scope;
    }, []);

    return ValidationError.of.apply(undefined, [name, value].concat(_toConsumableArray$1(types)));
  };

  return createContract(unionContract);
};

var isVoid = createContract(function (name, value) {
  return value === undefined ? value : ValidationError.of(name, value, 'void');
});

var isUndefined = isVoid;

exports.isArray = isArray;
exports.isLiteral = isLiteral;
exports.isNull = isNull;
exports.isNumber = isNumber;
exports.isObject = isObject;
exports.isString = isString;
exports.isUnion = isUnion;
exports.isUndefined = isUndefined;
exports.isVoid = isVoid;
exports.createContract = createContract;
exports.ValidationError = ValidationError;
